local GlobalAddonName, LTS = ...

local ADDON_VERSION = "1.0 (10.04.2016)"
--	Curr Legion Build 21414

local tonumber, GetItemCount, strtrim = tonumber, GetItemCount, strtrim

local CategoriesSlots = LTS.CategoriesSlots
local CategoriesSlotsByClass = LTS.CategoriesSlotsByClass
local CamerasRace = LTS.CamerasRace 
local SlotIcons = LTS.SlotIcons
local CameraData = LTS.CameraData
local CategoryBigModelItem = LTS.CategoryBigModelItem

local AllData
local CamerasData
local Visuals
local ItemSource
local VendorNames
local InstanceList
local BossList
local DiffList
local ItemCost
local ItemsCheckBonuses

local VLTW

local _,playerClass = UnitClass'player'
local _,playerRace = UnitRace'player'
local playerSex = UnitSex'player'

CategoriesSlotsByClass = CategoriesSlotsByClass[playerClass]

local realmKey = GetRealmName() or ""
local charName = UnitName'player' or ""
realmKey = realmKey:gsub(" ","")
local charKey = charName .. "-" .. realmKey
local charKeyFind = charKey:gsub("%-","%%%-")

local SourceTypeNames = {
	[1] = "Boss Drop",
	[2] = "Quest",
	[3] = "Vendor",
	[4] = "World Drop",
	[5] = "Legacy",
	[6] = "Unknown",
	[7] = "Achievement",
	[8] = "Profession",
}
local SourceTypeBitFilters = {
	[0] = 0,
	[1] = 1,
	[2] = 2,
	[3] = 4,
	[4] = 8,
	[5] = 16,
	[6] = 32,
	[7] = 64,
	[8] = 128,
}

local majorBonuses = {
	[451] = true,
	[450] = true,
	[566] = true,
	[567] = true,
	[524] = true,
}

local CameraDataAnimation = {
	--[8] = 15,
	--[14] = 15,
	[1] = 8,
	[11] = 8,
	[12] = 8,
}

if playerRace == "Gnome" and playerSex == 2 then	--Gnome Male Fix
	CameraDataAnimation[10] = 8
	CameraDataAnimation[13] = 8
	CameraDataAnimation[14] = 8
	CameraDataAnimation[15] = 8
	CameraDataAnimation[16] = 8
elseif playerRace == "Pandaren" and playerSex == 2 then	--Pandaren Male Fix
	CameraDataAnimation[15] = 8
end

local CameraDataOffset = 0
local currentPage, currentMaxPages, currentCategory = 1, 1
local CharQuestsData
local IsItemCollected, IsUsedZeroCam, IsFilterEnabled
local UpdatePage, UpdateCategory, UpdateCamera0, UpdateAlerts
local eventsFrame
local FilterBits, FilterBitsMax = 255, 255

if not GetBuildInfo():find("^6%.") then
	eventsFrame = CreateFrame("Frame")
	eventsFrame:RegisterEvent("ADDON_LOADED")
	eventsFrame:SetScript("OnEvent",function(self,event,arg1)
		if event == "ADDON_LOADED" then
			if arg1 == GlobalAddonName then
				if type(_G.VLTW)=='number' then
					eventsFrame:UnregisterEvent("ADDON_LOADED")
				end
			elseif arg1 == "Blizzard_Collections" then
				local thxFrame = CreateFrame("Frame",nil,CollectionsJournal)
				thxFrame:SetPoint("TOPLEFT",CollectionsJournal,"BOTTOMLEFT",0,-40)
				thxFrame:SetPoint("TOPRIGHT",CollectionsJournal,"BOTTOMRIGHT",0,-40)
				thxFrame:SetHeight(60)
				
				
				thxFrame:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",tile = true,tileSize = 1,edgeSize = 8,insets = {left = 2,right = 2,top = 2,bottom = 2}})
				thxFrame:SetBackdropColor(0, 0, 0, .8) 
				thxFrame:SetBackdropBorderColor(0.6, 0.6, 0.6, 1)
				
				local closeButton = CreateFrame("Button",nil,thxFrame,"UIPanelCloseButton")
				closeButton:SetPoint("TOPRIGHT",4,5)
				
				local thxText = thxFrame:CreateFontString(nil, "ARTWORK","GameFontWhite")
				thxText:SetPoint("TOPLEFT",10,-5)
				thxText:SetPoint("BOTTOMRIGHT",-10,5)
				thxText:SetJustifyH("LEFT")
				thxText:SetJustifyV("MIDDLE")
				thxText:SetText("Thanks for using <Legion Wardrobe> addon. Legion invasion already happened and wardrobe+updated transmog system were added to the game. GL & HF in Azeroth.")
				
				_G.VLTW = 1
			end
		end
	end)
	return
end

local function Model_ApplyUICamera(self, uiCameraID, disableOffset, disableAnimation)
	uiCameraID = uiCameraID or 1
	local currCameraData = CamerasData[uiCameraID + (CameraDataOffset * (disableOffset and 0 or 1))] or CamerasData[uiCameraID] or CamerasData[1]
	local posX, posY, posZ, facing = unpack(currCameraData)
	self:MakeCurrentCameraCustom()
	self:SetPosition(posX, posY, posZ)
	self:SetCameraPosition(1, 0, 0)
	self:SetCameraTarget(0, 0, 0)
	self:SetFacing(facing)
	if not disableAnimation then
		self:SetAnimation(CameraDataAnimation[uiCameraID % 20] or 15)
	end
	
	self.cameraID = uiCameraID
end

local function GetCursorPos(obj)
	local x_f,y_f = GetCursorPosition()
	local s = obj:GetEffectiveScale()
	x_f, y_f = x_f/s, y_f/s
	local x,y = obj:GetLeft(),obj:GetTop()
	x = x_f-x
	y = (y_f-y)*(-1)
	return x,y
end

local mainFrame = CreateFrame("Frame",nil,UIParent,"InsetFrameTemplate")
mainFrame:SetPoint("CENTER")
mainFrame:SetSize(700,600)
mainFrame:EnableMouse(true)
mainFrame:SetMovable(true)
mainFrame:SetClampedToScreen(true)
mainFrame:RegisterForDrag("LeftButton")
mainFrame:SetScript("OnDragStart", function(self) 
	if self:IsMovable() then 
		self:StartMoving() 
	end 
end)
mainFrame:SetScript("OnDragStop", function(self)
	self:StopMovingOrSizing()
end)
mainFrame:SetFrameStrata("HIGH")
mainFrame:Hide()

mainFrame.titleText = mainFrame:CreateFontString(nil, "OVERLAY","GameFontWhite", 2)
mainFrame.titleText:SetFont(mainFrame.titleText:GetFont(),12)
mainFrame.titleText:SetPoint("TOP",0,-4)
mainFrame.titleText:SetJustifyH("LEFT")
mainFrame.titleText:SetJustifyV("TOP")
mainFrame.titleText:SetText("Wardrobe")

do
	local OverlayShadowTopLeft = mainFrame:CreateTexture(nil,"OVERLAY")
	OverlayShadowTopLeft:SetPoint("TOPLEFT",4,-4)
	OverlayShadowTopLeft:SetAtlas("collections-background-shadow-small",true)

	local OverlayShadowTop = mainFrame:CreateTexture(nil,"OVERLAY")
	OverlayShadowTop:SetPoint("TOPLEFT",17,-4)
	OverlayShadowTop:SetPoint("TOPRIGHT",-17,-4)
	OverlayShadowTop:SetAtlas("collections-background-shadow-small",true)
	OverlayShadowTop:SetTexCoord(0.9999,1,0,1)
	
	local OverlayShadowTopRight = mainFrame:CreateTexture(nil,"OVERLAY")
	OverlayShadowTopRight:SetPoint("TOPRIGHT",-4,-4)
	OverlayShadowTopRight:SetAtlas("collections-background-shadow-small",true)
	OverlayShadowTopRight:SetTexCoord(1,0,0,1)

	local OverlayShadowLeft = mainFrame:CreateTexture(nil,"OVERLAY")
	OverlayShadowLeft:SetPoint("TOPLEFT",4,-17)
	OverlayShadowLeft:SetPoint("BOTTOMLEFT",4,17)
	OverlayShadowLeft:SetAtlas("collections-background-shadow-small",true)
	OverlayShadowLeft:SetTexCoord(0,1,0.9999,1)
	
	local OverlayShadowBottomLeft = mainFrame:CreateTexture(nil,"OVERLAY")
	OverlayShadowBottomLeft:SetPoint("BOTTOMLEFT",4,4)
	OverlayShadowBottomLeft:SetAtlas("collections-background-shadow-small",true)
	OverlayShadowBottomLeft:SetTexCoord(0,1,1,0)
	
	local OverlayShadowRight = mainFrame:CreateTexture(nil,"OVERLAY")
	OverlayShadowRight:SetPoint("TOPRIGHT",-4,-17)
	OverlayShadowRight:SetPoint("BOTTOMRIGHT",-4,17)
	OverlayShadowRight:SetAtlas("collections-background-shadow-small",true)
	OverlayShadowRight:SetTexCoord(1,0,0.9999,1)
	
	local OverlayShadowBottomRight = mainFrame:CreateTexture(nil,"OVERLAY")
	OverlayShadowBottomRight:SetPoint("BOTTOMRIGHT",-4,4)
	OverlayShadowBottomRight:SetAtlas("collections-background-shadow-small",true)
	OverlayShadowBottomRight:SetTexCoord(1,0,1,0)
	
	local OverlayShadowBottom = mainFrame:CreateTexture(nil,"OVERLAY")
	OverlayShadowBottom:SetPoint("BOTTOMLEFT",17,4)
	OverlayShadowBottom:SetPoint("BOTTOMRIGHT",-17,4)
	OverlayShadowBottom:SetAtlas("collections-background-shadow-small",true)
	OverlayShadowBottom:SetTexCoord(0.9999,1,1,0)


	local ShadowLineTop = mainFrame:CreateTexture(nil,"BORDER",nil,1)
	ShadowLineTop:SetPoint("TOPLEFT",4,-13)
	ShadowLineTop:SetPoint("BOTTOMRIGHT",mainFrame,"TOPRIGHT",-4,-17)
	ShadowLineTop:SetAtlas("_collections-background-line",true)
	ShadowLineTop:SetHorizTile(true)

	local ShadowLineBottom = mainFrame:CreateTexture(nil,"BORDER",nil,1)
	ShadowLineBottom:SetPoint("TOPLEFT",mainFrame,"BOTTOMLEFT",4,17)
	ShadowLineBottom:SetPoint("BOTTOMRIGHT",-4,13)
	ShadowLineBottom:SetAtlas("_collections-background-line",true)
	ShadowLineBottom:SetHorizTile(true)
	
	local ShadowCornerTopLeft = mainFrame:CreateTexture(nil,"BORDER",nil,2)
	ShadowCornerTopLeft:SetPoint("TOPLEFT",4,-4)
	ShadowCornerTopLeft:SetAtlas("collections-background-shadow-large",true)
	
	local ShadowCornerTopRight = mainFrame:CreateTexture(nil,"BORDER",nil,2)
	ShadowCornerTopRight:SetPoint("TOPRIGHT",-4,-4)
	ShadowCornerTopRight:SetAtlas("collections-background-shadow-large",true)
	ShadowCornerTopRight:SetTexCoord(1,0,0,1)
	
	local ShadowCornerBottomLeft = mainFrame:CreateTexture(nil,"BORDER",nil,2)
	ShadowCornerBottomLeft:SetPoint("BOTTOMLEFT",4,4)
	ShadowCornerBottomLeft:SetAtlas("collections-background-shadow-large",true)
	ShadowCornerBottomLeft:SetTexCoord(0,1,1,0)
	
	local ShadowCornerBottomRight = mainFrame:CreateTexture(nil,"BORDER",nil,2)
	ShadowCornerBottomRight:SetPoint("BOTTOMRIGHT",-4,4)
	ShadowCornerBottomRight:SetAtlas("collections-background-shadow-large",true)
	ShadowCornerBottomRight:SetTexCoord(1,0,1,0)

	local ShadowCornerTop = mainFrame:CreateTexture(nil,"BORDER",nil,2)
	ShadowCornerTop:SetPoint("TOPLEFT",149,-4)
	ShadowCornerTop:SetPoint("TOPRIGHT",-149,-4)
	ShadowCornerTop:SetAtlas("collections-background-shadow-large",true)
	ShadowCornerTop:SetTexCoord(0.9999,1,0,1)	
	
	local ShadowCornerLeft = mainFrame:CreateTexture(nil,"BORDER",nil,2)
	ShadowCornerLeft:SetPoint("TOPLEFT",4,-151)
	ShadowCornerLeft:SetPoint("BOTTOMLEFT",4,151)
	ShadowCornerLeft:SetAtlas("collections-background-shadow-large",true)
	ShadowCornerLeft:SetTexCoord(0,1,0.9999,1)

	local ShadowCornerRight = mainFrame:CreateTexture(nil,"BORDER",nil,2)
	ShadowCornerRight:SetPoint("TOPRIGHT",-4,-151)
	ShadowCornerRight:SetPoint("BOTTOMRIGHT",-4,151)
	ShadowCornerRight:SetAtlas("collections-background-shadow-large",true)
	ShadowCornerRight:SetTexCoord(1,0,0.9999,1)

	local ShadowCornerBottom = mainFrame:CreateTexture(nil,"BORDER",nil,2)
	ShadowCornerBottom:SetPoint("BOTTOMLEFT",149,4)
	ShadowCornerBottom:SetPoint("BOTTOMRIGHT",-149,4)
	ShadowCornerBottom:SetAtlas("collections-background-shadow-large",true)
	ShadowCornerBottom:SetTexCoord(0.9999,1,1,0)


	local BackgroundTile = mainFrame:CreateTexture(nil,"BACKGROUND")
	BackgroundTile:SetPoint("TOPLEFT",4,-4)
	BackgroundTile:SetPoint("BOTTOMRIGHT",-4,4)
	BackgroundTile:SetAtlas("collections-background-tile",true)
	BackgroundTile:SetVertTile(true)
	BackgroundTile:SetHorizTile(true)
end

local SlotsFrame = CreateFrame("Frame",nil,mainFrame)
SlotsFrame:SetSize(10,10)
SlotsFrame:SetPoint("TOPLEFT",18,-25)

do
	local lastButton = nil
	local buttonsList = {}
	local function ButtonOnClick(self)
		for i=1,#buttonsList do
			buttonsList[i].selected:Hide()
		end
		self.selected:Show()
		UpdateCategory(self._id)
	end
	local function ButtonOnEnter(self)
		if not self.tooltip then return end
		GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
		GameTooltip:AddLine(self.tooltip)
		GameTooltip:Show()
	end
	local function ButtonOnLeave(self)
		GameTooltip_Hide()
	end
	for i=1,#CategoriesSlotsByClass do
		local CategoryID = CategoriesSlotsByClass[i]
		local CategoryName = CategoriesSlots[CategoryID]
	
		local button = CreateFrame("Button",nil,SlotsFrame)
		if lastButton then
			if i == 21 then
				button:SetPoint("TOP", lastButton, "BOTTOM", 0, -5)
			elseif i > 21 then
				button:SetPoint("RIGHT", lastButton, "LEFT", -2, 0)
			else
				button:SetPoint("LEFT", lastButton, "RIGHT", 2, 0)
			end
		else
			button:SetPoint("TOPLEFT")
		end
		button:SetSize(31,31)
		button:SetScript("OnClick",ButtonOnClick)
		button:SetScript("OnEnter",ButtonOnEnter)
		button:SetScript("OnLeave",ButtonOnLeave)
		button._id = CategoryID
		button.tooltip = CategoryName
		lastButton = button
		buttonsList[#buttonsList + 1] = button
		
		local normalTexture = button:CreateTexture(nil,"BACKGROUND")
		normalTexture:SetAllPoints()
		normalTexture:SetTexture([[Interface\AddOns\LegionWardrobe\Transmogrify]])
		local iconTCoord = SlotIcons[ CategoryID ]
		normalTexture:SetTexCoord(iconTCoord[1]/512,iconTCoord[2]/512,iconTCoord[3]/512,iconTCoord[4]/512)
		
		local highlightTexture = button:CreateTexture(nil,"BACKGROUND")
		highlightTexture:SetSize(31,31)
		highlightTexture:SetAtlas("bags-roundhighlight")
		highlightTexture:SetPoint("CENTER",0,1)
		
		local selectedTexture = button:CreateTexture(nil,"OVERLAY")
		selectedTexture:SetSize(45,45)
		selectedTexture:SetTexture([[Interface\AddOns\LegionWardrobe\Transmogrify]])
		selectedTexture:SetPoint("CENTER")
		local iconTCoord = SlotIcons[ -1 ]
		selectedTexture:SetTexCoord(iconTCoord[1]/512,iconTCoord[2]/512,iconTCoord[3]/512,iconTCoord[4]/512)
		selectedTexture:SetShown(i==1)
		button.selected = selectedTexture
		
		button:SetNormalTexture(normalTexture) 
		button:SetHighlightTexture(highlightTexture,"ADD") 
	end
	
	
	local bigModelButton = CreateFrame("Button",nil,SlotsFrame)
	bigModelButton:SetPoint("TOPLEFT", buttonsList[1], "BOTTOMLEFT", 0, -5)
	bigModelButton:SetSize(31,31)
	bigModelButton:SetScript("OnClick",function(self)
		if mainFrame.bigModel:IsShown() then
			mainFrame.bigModel:Hide()
			self.Texture:SetDesaturated(true)
		else
			mainFrame.bigModel:Show()
			self.Texture:SetDesaturated(false)
		end
	end)
	bigModelButton:SetScript("OnEnter",ButtonOnEnter)
	bigModelButton:SetScript("OnLeave",ButtonOnLeave)
	bigModelButton.tooltip = "Open Big Model"
	
	bigModelButton.Texture = bigModelButton:CreateTexture(nil,"BACKGROUND")
	bigModelButton.Texture:SetAllPoints()
	bigModelButton.Texture:SetTexture([[Interface\AddOns\LegionWardrobe\Transmogrify]])
	bigModelButton.Texture:SetTexCoord(450/512,482/512,88/512,120/512)
	bigModelButton.Texture:SetDesaturated(true)
end

local ProgressBar = CreateFrame("StatusBar",nil,mainFrame)
ProgressBar:SetSize(196,13)
ProgressBar:SetPoint("TOP",0,-70)
ProgressBar:SetStatusBarTexture([[Interface\PaperDollInfoFrame\UI-Character-Skills-Bar]]) 
ProgressBar:SetStatusBarColor(0.03125,0.85,0.0)

do
	local borderLeft = ProgressBar:CreateTexture(nil,"OVERLAY")
	borderLeft:SetPoint("LEFT",-3,0)
	borderLeft:SetSize(8,17)
	borderLeft:SetTexture([[Interface\PaperDollInfoFrame\UI-Character-Skills-BarBorder]])
	borderLeft:SetTexCoord(0.007843,0.043137,0.193548,0.774193)

	local borderRight = ProgressBar:CreateTexture(nil,"OVERLAY")
	borderRight:SetPoint("RIGHT",4,0)
	borderRight:SetSize(8,17)
	borderRight:SetTexture([[Interface\PaperDollInfoFrame\UI-Character-Skills-BarBorder]])
	borderRight:SetTexCoord(0.043137,0.007843,0.193548,0.774193)

	local borderMid = ProgressBar:CreateTexture(nil,"OVERLAY")
	borderMid:SetPoint("TOPLEFT",borderLeft,"TOPRIGHT")
	borderMid:SetPoint("BOTTOMRIGHT",borderRight,"BOTTOMLEFT")
	borderMid:SetTexture([[Interface\PaperDollInfoFrame\UI-Character-Skills-BarBorder]])
	borderMid:SetTexCoord(0.113726,0.1490196,0.193548,0.774193)
	
	local Background = ProgressBar:CreateTexture(nil,"BACKGROUND",nil,-1)
	Background:SetAllPoints()
	Background:SetTexture(0,0,0,1)
	
	local text = ProgressBar:CreateFontString(nil,"OVERLAY","TextStatusBarText",1)
	text:SetPoint("CENTER",0,0)
	text:SetSize(0,9)
	text:SetJustifyH("CENTER")
	ProgressBar.text = text
end


local closeButton = CreateFrame("Button",nil,mainFrame,"UIPanelCloseButton")
closeButton:SetPoint("TOPRIGHT",4,5)

do
	local model = CreateFrame("DressUpModel",nil,mainFrame)
	model:SetPoint("TOPRIGHT",mainFrame,"TOPLEFT",-10,0)
	
	mainFrame.bigModel = model
	
	model:SetSize(mainFrame:GetHeight() * 90 / 130,mainFrame:GetHeight())
	model:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",tile = true,tileSize = 1,edgeSize = 8,insets = {left = 2,right = 2,top = 2,bottom = 2}})
	model:SetBackdropColor(0, 0, 0, 0.8) 
	model:SetBackdropBorderColor(0.6, 0.6, 0.6, 1) 
	
	model:MakeCurrentCameraCustom()
	model.cameraID = 1
	
	model:Hide()
	
	function UpdateCamera0()
		local cx,cy,cz = model:GetPosition()
		local cf = model:GetFacing()
		
		IsUsedZeroCam = true
		
		CamerasData[0] = CamerasData[0] or {}
		CamerasData[0][1] = cx
		CamerasData[0][2] = cy
		CamerasData[0][3] = cz
		CamerasData[0][4] = cf
		
		for i=1,#mainFrame.models do
			Model_ApplyUICamera(mainFrame.models[i], 0, true, true)
		end
	end
	
	model:EnableMouse(true)
	model:SetScript("OnMouseDown",function (self,button)
		local x,y = GetCursorPos(self)
		self.x = x
		self.y = y
		local cx,cy,cz = self:GetPosition()
		self.cx = cx
		self.cy = cy
		self.cz = cz
		self.cf = self:GetFacing()
		self.mov = true
		self.isRotation = button == "RightButton"
		
		self.TextInfo:Hide()
	end)
	model:SetScript("OnMouseUp",function (self)
		UpdateCamera0()
		self.mov = nil
	end)
	model:SetScript("OnUpdate",function (self)
		if not self.mov then
			return
		end
		local x,y = GetCursorPos(self)
		local diff_x = x - self.x
		local diff_y = y - self.y
		if self.isRotation then
			self:SetFacing(self.cf + diff_x / 65)
		else
			self:SetPosition(self.cx,self.cy + diff_x / 200,self.cz - diff_y / 200)
		end
		UpdateCamera0()
	end)
	model:SetScript("OnMouseWheel",function (self,delta)
		local cx,cy,cz = self:GetPosition()
		self:SetPosition(cx + delta / 20,cy,cz)
		UpdateCamera0()
		
		self.TextInfo:Hide()
	end)
	model:SetScript("OnModelLoaded",function (self)
		Model_ApplyUICamera(self, self.cameraID)
	end)
	
	local function Reload(self)
		self:SetUnit("player")
		self:Undress()
		self:TryOn(self.itemID or "item:94984")
		
		Model_ApplyUICamera(self, self.cameraID)
	end
	model.Reload = Reload
	
	model:SetScript("OnShow",function (self)
		self:Reload()
	end)
	
	model.TextInfo = model:CreateFontString(nil, "OVERLAY","GameFontWhite", 1)
	model.TextInfo:SetFont(model.TextInfo:GetFont(),11)
	model.TextInfo:SetPoint("TOPLEFT",4,-4)
	model.TextInfo:SetJustifyH("LEFT")
	model.TextInfo:SetJustifyV("TOP")
	model.TextInfo:SetText("Left Click+Move - moving\nRight Click+Move - rotation\nMouse Wheel - scaling")
end

do
	local alert = CreateFrame("Frame",nil,mainFrame)
	alert:SetSize(1,1)
	alert:SetPoint("BOTTOMLEFT",20,8)
	alert:Hide()
	
	alert.icon = alert:CreateTexture(nil,"BACKGROUND")
	alert.icon:SetSize(24,24)
	alert.icon:SetPoint("BOTTOMLEFT",0,0)
	alert.icon:SetTexture("Interface\\DialogFrame\\UI-Dialog-Icon-AlertNew")
	
	alert.text = alert:CreateFontString(nil, "ARTWORK","GameFontWhite")
	alert.text:SetPoint("BOTTOMLEFT",26,0)
	alert.text:SetHeight(24)
	alert.text:SetJustifyH("LEFT")
	alert.text:SetJustifyV("MIDDLE")
	alert.text:SetText("")
	
	function UpdateAlerts(showVSalert, showBankAlert)
		if VLTW._AlertsOff then
			alert:Hide()
		elseif showVSalert and showBankAlert then
			alert:Show()
			alert.text:SetText("Void Storage data isn't collected; Bank data isn't collected (for WoD items)")
		elseif showVSalert then
			alert:Show()
			alert.text:SetText("Void Storage data isn't collected")
		elseif showBankAlert then
			alert:Show()
			alert.text:SetText("Bank data isn't collected (for WoD items)")
		else
			alert:Hide()
		end
	end
end

do
	local aboutFrameBack = CreateFrame("Button",nil,mainFrame)
	aboutFrameBack:Hide()

	aboutFrameBack:SetPoint("TOPLEFT")
	aboutFrameBack:SetPoint("BOTTOMRIGHT")
	
	aboutFrameBack.Texture = aboutFrameBack:CreateTexture(nil,"OVERLAY",nil,3)
	aboutFrameBack.Texture:SetAllPoints()
	aboutFrameBack.Texture:SetTexture(0,0,0,.8)
	
	aboutFrameBack:SetScript("OnClick",function() end)
	
	aboutFrameBack:SetFrameStrata("DIALOG")

	local aboutFrame = CreateFrame("Frame",nil,aboutFrameBack)
	aboutFrame:SetPoint("CENTER")
	aboutFrame:SetSize(650,220)
	
	aboutFrame:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",tile = true,tileSize = 1,edgeSize = 8,insets = {left = 2,right = 2,top = 2,bottom = 2}})
	aboutFrame:SetBackdropColor(1, 1, .8, 1) 
	aboutFrame:SetBackdropBorderColor(0.6, 0.6, 0.6, 1)
	
	local closeButton = CreateFrame("Button",nil,aboutFrame,"UIPanelCloseButton")
	closeButton:SetPoint("TOPRIGHT",3,2)
	closeButton:SetScript("OnClick",function()
		aboutFrameBack:Hide()
	end)
	
	local authorText = aboutFrame:CreateFontString(nil, "ARTWORK","GameFontWhite")
	authorText:SetPoint("TOPLEFT",10,-10)
	authorText:SetJustifyH("LEFT")
	authorText:SetJustifyV("TOP")
	authorText:SetShadowColor(0,0,0,1)
	authorText:SetShadowOffset(1,-1)
	authorText:SetText("Author: Afiya @ EU-Howling Fjord")

	local versionText = aboutFrame:CreateFontString(nil, "ARTWORK","GameFontWhite")
	versionText:SetPoint("TOPLEFT",10,-25)
	versionText:SetJustifyH("LEFT")
	versionText:SetJustifyV("TOP")
	versionText:SetShadowColor(0,0,0,1)
	versionText:SetShadowOffset(1,-1)
	versionText:SetText("Addon Version: "..ADDON_VERSION)		
	
	local ciit = aboutFrame:CreateFontString(nil, "ARTWORK","GameFontWhite")
	ciit:SetPoint("TOPLEFT",10,-55)
	ciit:SetJustifyH("LEFT")
	ciit:SetJustifyV("TOP")
	ciit:SetShadowColor(0,0,0,1)
	ciit:SetShadowOffset(1,-1)
	ciit:SetText("Collected icons info:")
	
	local gie = aboutFrame:CreateTexture(nil,"BACKGROUND")
	gie:SetPoint("TOPLEFT",ciit,"BOTTOMLEFT",5,-5)
	gie:SetSize(18,18)
	gie:SetTexture("Interface\\RaidFrame\\ReadyCheck-Ready")

	local giet = aboutFrame:CreateFontString(nil, "ARTWORK","GameFontWhite")
	giet:SetPoint("LEFT",gie,"RIGHT",5,0)
	giet:SetHeight(18)
	giet:SetJustifyH("LEFT")
	giet:SetJustifyV("MIDDLE")
	giet:SetShadowColor(0,0,0,1)
	giet:SetShadowOffset(1,-1)
	giet:SetText("You have collected item on current character")	

	local yie = aboutFrame:CreateTexture(nil,"BACKGROUND")
	yie:SetPoint("TOPLEFT",gie,"BOTTOMLEFT",0,-2)
	yie:SetSize(18,18)
	yie:SetTexture([[Interface\AddOns\LegionWardrobe\Transmogrify]])
	yie:SetTexCoord(222/512,250/512,127/512,153/512)

	local yiet = aboutFrame:CreateFontString(nil, "ARTWORK","GameFontWhite")
	yiet:SetPoint("LEFT",yie,"RIGHT",5,0)
	yiet:SetHeight(18)
	yiet:SetJustifyH("LEFT")
	yiet:SetJustifyV("MIDDLE")
	yiet:SetShadowColor(0,0,0,1)
	yiet:SetShadowOffset(1,-1)
	yiet:SetText("You have collected item on another character *")	

	local nt = aboutFrame:CreateFontString(nil, "ARTWORK","GameFontWhite")
	nt:SetPoint("TOP",yie,"BOTTOM",0,-5)
	nt:SetPoint("LEFT",ciit,0,0)
	nt:SetPoint("RIGHT",aboutFrame,-10,0)
	nt:SetJustifyH("LEFT")
	nt:SetJustifyV("TOP")
	nt:SetShadowColor(0,0,0,1)
	nt:SetShadowOffset(1,-1)
	nt:SetText("* Note: Collecting items via quests will unlock only items that can be equipped by current character (for armors it must be of your primary armor type).")
	
	local disableOtherChars = CreateFrame("CheckButton",nil,aboutFrame,"UICheckButtonTemplate")  
	disableOtherChars.text:SetText("|cffffffffDisable tracking items from different characters")
	disableOtherChars:SetPoint("TOP",nt,"BOTTOM",0,-10)
	disableOtherChars:SetPoint("LEFT",aboutFrame,10,0)
	disableOtherChars:SetScript("OnClick",function (self)
		VLTW._Disable = self:GetChecked()
		UpdateCategory(currentCategory)
	end)
		
	mainFrame.disableOtherChars = disableOtherChars
	
	local disableAlerts = CreateFrame("CheckButton",nil,aboutFrame,"UICheckButtonTemplate")  
	disableAlerts.text:SetText("|cffffffffDisable alerts about Void Storage and Bank")
	disableAlerts:SetPoint("TOPLEFT",disableOtherChars,"BOTTOMLEFT",0,5)
	disableAlerts:SetScript("OnClick",function (self)
		VLTW._AlertsOff = self:GetChecked()
		UpdateAlerts(not VLTW[charKey].visited_VS,not VLTW[charKey].visited_Bank)
	end)
	
	mainFrame.disableAlerts = disableAlerts

	
	local function ButtonOnEnter(self)
		if not self.tooltip then return end
		GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
		GameTooltip:AddLine(self.tooltip)
		GameTooltip:Show()
	end
	local function ButtonOnLeave(self)
		GameTooltip_Hide()
	end
	local function ButtonOnClick(self)
		aboutFrameBack:Show()
	end

	local button = CreateFrame("Button",nil,mainFrame)
	button:SetPoint("BOTTOMRIGHT",-20,13)
	button:SetSize(31,31)
	button:SetScript("OnClick",ButtonOnClick)
	button:SetScript("OnEnter",ButtonOnEnter)
	button:SetScript("OnLeave",ButtonOnLeave)
	button.tooltip = "About & Help"
	
	local normalTexture = button:CreateTexture(nil,"BACKGROUND")
	normalTexture:SetAllPoints()
	normalTexture:SetTexture([[Interface\AddOns\LegionWardrobe\Transmogrify]])
	normalTexture:SetTexCoord(191/512,220/512,127/512,156/512)
	
	local highlightTexture = button:CreateTexture(nil,"BACKGROUND")
	highlightTexture:SetSize(31,31)
	highlightTexture:SetAtlas("bags-roundhighlight")
	highlightTexture:SetPoint("CENTER",0,1)
	
	button:SetNormalTexture(normalTexture) 
	button:SetHighlightTexture(highlightTexture,"ADD") 
end

do

	local FilterButton = CreateFrame("Button",nil,mainFrame,"UIMenuButtonStretchTemplate")
	FilterButton:SetPoint("BOTTOMRIGHT",-55,20)
	FilterButton:SetText(FILTER)
	FilterButton:SetSize(93,22)
	
	local FilterButtonIcon = FilterButton:CreateTexture(nil,"ARTWORK")
	FilterButtonIcon:SetPoint("RIGHT",-5,0)
	FilterButtonIcon:SetSize(10,12)
	FilterButtonIcon:SetTexture([[Interface\ChatFrame\ChatFrameExpandArrow]])

	local FilterDropDown = CreateFrame("Button",GlobalAddonName.."FilterDropDown",mainFrame,"UIDropDownMenuTemplate")
	UIDropDownMenu_Initialize(FilterDropDown, function(self, level)
		local info = UIDropDownMenu_CreateInfo()
		--info.keepShownOnClick = true
		if level == 1 then
			info.hasArrow = false
			info.isNotRadio = true
			info.notCheckable = true

			info.text = CHECK_ALL
			info.func = function()
				FilterBits = 0
				IsFilterEnabled = false
				for _,sbit in pairs(SourceTypeBitFilters) do
					FilterBits = bit.bor(FilterBits,sbit)
				end
				UIDropDownMenu_Refresh(FilterDropDown, 1, 2)
				UpdateCategory(currentCategory)
			end
			UIDropDownMenu_AddButton(info, level)
			
			info.text = UNCHECK_ALL
			info.func = function()
				FilterBits = 0
				IsFilterEnabled = true
				UIDropDownMenu_Refresh(FilterDropDown, 1, 2)
				UpdateCategory(currentCategory)
			end
			UIDropDownMenu_AddButton(info, level)
			info.notCheckable = false

			for i = 1, #SourceTypeNames do
				info.text = SourceTypeNames[i]
				info.func = function(_, _, _, value)
					if value then
						FilterBits = bit.bxor(FilterBits,SourceTypeBitFilters[i])
					else
						FilterBits = bit.bor(FilterBits,SourceTypeBitFilters[i])
					end
					IsFilterEnabled = not (FilterBits == FilterBitsMax)
					UpdateCategory(currentCategory)
				end
				info.checked = function() return bit.band(SourceTypeBitFilters[i],FilterBits) > 0 end
				UIDropDownMenu_AddButton(info, level)
			end
		end
	end, "MENU")

	FilterButton:SetScript("OnClick",function (self)
		PlaySound("igMainMenuOptionCheckBoxOn")
		ToggleDropDownMenu(1, nil, FilterDropDown, self, 74, 15)
	end)
	
end

local inspectScantip = CreateFrame("GameTooltip", GlobalAddonName.."ScanningTooltip", nil, "GameTooltipTemplate")
inspectScantip:SetOwner(UIParent, "ANCHOR_NONE")

local sourcesFrame = CreateFrame("Frame",GlobalAddonName.."SourcesFrame",UIParent,"UIPanelDialogTemplate")
sourcesFrame:SetSize(730,300)
sourcesFrame:SetPoint("CENTER")
sourcesFrame:SetFrameStrata("DIALOG")
sourcesFrame:SetClampedToScreen(true)
sourcesFrame:EnableMouse(true)
sourcesFrame:SetMovable(true)
sourcesFrame:RegisterForDrag("LeftButton")
sourcesFrame:SetScript("OnDragStart", function(self) 
	self:StartMoving() 
end)
sourcesFrame:SetScript("OnDragStop", function(self) 
	self:StopMovingOrSizing() 
end)
sourcesFrame.title:SetText("Sources")
sourcesFrame:Hide()

sourcesFrame.TextLine = sourcesFrame:CreateFontString(nil,"ARTWORK","GameFontWhite")
sourcesFrame.TextLine:SetPoint("TOPLEFT",25,-30)
sourcesFrame.TextLine:SetSize(0,20)
sourcesFrame.TextLine:SetJustifyH("LEFT")
sourcesFrame.TextLine:SetJustifyV("MIDDLE")

sourcesFrame.moreInfo = CreateFrame("Frame",nil,sourcesFrame)
sourcesFrame.moreInfo:Hide()
sourcesFrame.moreInfo:SetPoint("TOPLEFT",sourcesFrame,"TOPRIGHT",0,-25)
sourcesFrame.moreInfo:SetSize(220,300)
sourcesFrame.moreInfo:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",tile = true,tileSize = 1,edgeSize = 8,insets = {left = 2,right = 2,top = 2,bottom = 2}})
sourcesFrame.moreInfo:SetBackdropColor(0, 0, 0, 0.8) 
sourcesFrame.moreInfo:SetBackdropBorderColor(0.6, 0.6, 0.6, 1) 

local function SourcesFrame_EditBox_OnEscapePressed(self)
	self:ClearFocus()
end
local function SourcesFrame_HideTooltip(self)
	GameTooltip_Hide()
end
local function SourcesFrame_ItemName_OnEnter(self)
	if not self.link then return end
	GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
	GameTooltip:SetHyperlink(self.link)
	GameTooltip:Show()
end
local function SourcesFrame_SourceData_OnEnter(self)
	if not self.type then return end
	
	GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
	if self.type == 2 then
		GameTooltip:AddLine(self.sourceType)
		for i=2,#self.quests do
			local questID = self.quests[i]
			local questName
			inspectScantip:SetHyperlink("quest:"..questID)
			if inspectScantip:NumLines() > 0 then
				local tooltipLine = _G[GlobalAddonName.."ScanningTooltipTextLeft1"]
				local text = tooltipLine:GetText()
				if text and text ~= "" then
					questName = text
				end
			end
			inspectScantip:ClearLines()
			if questName then
				GameTooltip:AddLine(questName)
			else
				GameTooltip:AddLine("Quest: "..questID)
			end
		end
	elseif self.type == 7 or self.type == 8 then
		GameTooltip:SetHyperlink(self.link)
	elseif self.type == 3 then
		GameTooltip:AddLine("Cost")
		local data = self.data
		if data[1] == 1 then
			local amount = GetCoinTextureString(data[2])
			GameTooltip:AddLine(amount)
		elseif data[1] == 2 then
			for i=2,#data,2 do
				local name, _, texturePath = GetCurrencyInfo(data[i])
				GameTooltip:AddLine(name..": "..data[i+1])
				GameTooltip:AddTexture(texturePath)
			end
		elseif data[1] == 3 then
			for i=2,#data,2 do
				local _, link, _, _, _, _, _, _, _, texturePath = GetItemInfo(data[i])
				if link then
					GameTooltip:AddLine(link..": "..data[i+1])
					GameTooltip:AddTexture(texturePath)
				end
			end
		end
	else
		GameTooltip:AddLine(self.sourceType)
		GameTooltip:AddLine(self.instanceType == 2 and "Raid" or "Instance")
		GameTooltip:AddLine("Zone: |cffffffff"..InstanceList[self.instanceID].."|r")
		GameTooltip:AddLine("Boss: |cffffffff"..BossList[self.bossID].."|r")
		if #self.diffList > 1 then
			GameTooltip:AddLine("Difficulties:")
			for i=1,#self.diffList do
				GameTooltip:AddLine("- |cffffffff"..DiffList[tonumber(self.diffList[i])].."|r")
			end
		elseif self.diffList[1] == "" then
			--nothing
		else
			GameTooltip:AddLine("Difficulty: |cffffffff"..DiffList[tonumber(self.diffList[1])].."|r")
		end
	end
	GameTooltip:Show()
end


local InputFrame = nil
local function ShowInputFrame(title,inputText)
	if not InputFrame then
		InputFrame = CreateFrame("Frame",GlobalAddonName.."ShowInputFrame",UIParent,"UIPanelDialogTemplate")
		InputFrame:SetSize(500,100)
		InputFrame:SetPoint("CENTER")
		InputFrame:SetFrameStrata("FULLSCREEN")
		InputFrame:SetClampedToScreen(true)
		InputFrame:EnableMouse(true)
		InputFrame:SetMovable(true)
		InputFrame:RegisterForDrag("LeftButton")
		InputFrame:SetScript("OnDragStart", function(self) 
			self:StartMoving() 
		end)
		InputFrame:SetScript("OnDragStop", function(self) 
			self:StopMovingOrSizing() 
		end)
		InputFrame:SetScript("OnShow",function (self)
			self.EditBox:HighlightText()
			self.EditBox:SetFocus()
		end)
		InputFrame:Hide()
	
		InputFrame.EditBox = CreateFrame("EditBox",GlobalAddonName.."ShowInputFrameEditBox",InputFrame,"InputBoxTemplate")
		InputFrame.EditBox:SetSize(470,20)
		InputFrame.EditBox:SetPoint("TOP",5,-35)
		InputFrame.EditBox:SetCursorPosition(0)
		InputFrame.EditBox:SetAutoFocus( false )
		InputFrame.EditBox:SetScript("OnEscapePressed",function(self) 
			self:ClearFocus() 
			self:GetParent().OK:Click("LeftButton")
		end)
		InputFrame.EditBox:SetScript("OnEnterPressed",function (self)
			self:GetParent().OK:Click("LeftButton")
		end)
		
		InputFrame.OK = CreateFrame("Button",nil,InputFrame,"UIPanelButtonTemplate")
		InputFrame.OK:SetPoint("TOP",0,-60)
		InputFrame.OK:SetSize(130,22)
		InputFrame.OK:SetText(ACCEPT)
		InputFrame.OK:SetScript("OnClick",function (self)
			InputFrame:Hide()
			local input = InputFrame.EditBox:GetText()
		end)
	end
	InputFrame.title:SetText(title or "")
	InputFrame.EditBox:SetText(inputText or "")
	InputFrame:Show()
end


local function SourcesFrame_ItemLink_OnClick(self)
	ShowInputFrame("url",self.link)
end
local function SourcesFrame_MoreInfo_Text_OnEnter(self)
	if not self.type then return end
	GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
	if self.type == 2 then
		GameTooltip:SetHyperlink(self.data)
	elseif self.type == 3 then
		GameTooltip:AddLine("Cost")
		local data = self.data
		if data[1] == 1 then
			local amount = GetCoinTextureString(data[2])
			GameTooltip:AddLine(amount)
		elseif data[1] == 2 then
			for i=2,#data,2 do
				local name, _, texturePath = GetCurrencyInfo(data[i])
				GameTooltip:AddLine(name..": "..data[i+1])
				GameTooltip:AddTexture(texturePath)
			end
		elseif data[1] == 3 then
			for i=2,#data,2 do
				local _, link, _, _, _, _, _, _, _, texturePath = GetItemInfo(data[i])
				if link then
					GameTooltip:AddLine(link..": "..data[i+1])
					GameTooltip:AddTexture(texturePath)
				end
			end
		end
	else
		local data = self.data
		GameTooltip:AddLine(data[1] == 2 and "Raid" or "Instance")
		GameTooltip:AddLine("Zone: |cffffffff"..InstanceList[ data[2] ].."|r")
		GameTooltip:AddLine("Boss: |cffffffff"..BossList[ data[3] ].."|r")
		if #data[4] > 1 then
			GameTooltip:AddLine("Difficulties:")
			for i=1,#data[4] do
				GameTooltip:AddLine("- |cffffffff"..DiffList[tonumber(data[4][i])].."|r")
			end
		elseif data[4][1] == "" then
			--nothing
		else
			GameTooltip:AddLine("Difficulty: |cffffffff"..DiffList[tonumber(data[4][1])].."|r")
		end
	end
	GameTooltip:Show()
end

local function SourcesFrame_ItemName_OnClick(self)
	local link = self.link
	if not link then
		return
	elseif IsModifiedClick("CHATLINK") then
		if ChatEdit_GetActiveWindow() then
			ChatEdit_InsertLink(link)
		else
			ChatFrame_OpenChat(link)
		end
	elseif IsModifiedClick("DRESSUP") then
		DressUpItemLink(link)
	end
end

local SourceFrame_MoreInfo_CreateLine

local function SourcesFrame_MoreInfo_OnClick(self)
	if self.type == 2 then
		--quests
		for i=2,#self.data do
			local questID = self.data[i]
			local questName
			inspectScantip:SetHyperlink("quest:"..questID)
			if inspectScantip:NumLines() > 0 then
				local tooltipLine = _G["LegionWardrobeScanningTooltipTextLeft1"]
				local text = tooltipLine:GetText()
				if text and text ~= "" then
					questName = text
				end
			end
			inspectScantip:ClearLines()
			
			local line = SourceFrame_MoreInfo_CreateLine(i-1)
			if questName then
				line.text:SetText((CharQuestsData[questID] and "|cff00ff00" or "") .. questName)
			else
				line.text:SetText((CharQuestsData[questID] and "|cff00ff00" or "") .."Quest: "..questID)
			end
			line.textFrame.type = 2
			line.textFrame.data = "quest:"..questID
			line.link.link = "http://wowhead.com/quest="..questID
			line:Show()
		end
	elseif self.type == 3 then
		-- vendors
		for i=2,#self.data do
			local npcID = self.data[i]
			local line = SourceFrame_MoreInfo_CreateLine(i-1)
			line.text:SetText(VendorNames[npcID])
			line.link.link = "http://wowhead.com/npc="..npcID
			line.textFrame.type = nil
			if self.itemID  then
				local costTable = ItemCost[ self.itemID ]
				if costTable then
					costTable = costTable[npcID] or costTable
					line.textFrame.type = 3
					line.textFrame.data = costTable
					if costTable[1] == 3 then
						for j=2,#costTable,2 do
							GetItemInfo(costTable[j])
						end
					end
				end
			end
			line:Show()
		end
	elseif self.type == 1 then
		-- boss drop
		for i=2,#self.data do
			local instanceType,instanceID,bossID,diffList = self.data[i]:match("^(%d+)#(%d+)#(%d+)#(.*)")
			bossID = tonumber(bossID)
			
			local line = SourceFrame_MoreInfo_CreateLine(i-1)
			line.text:SetText(BossList[bossID])
			
			line.textFrame.type = 1
			line.textFrame.data = {
				tonumber(instanceType),
				tonumber(instanceID),
				bossID,
				{strsplit(":", diffList)},
			}
			line.link.link = "http://wowhead.com/search?q="..BossList[bossID]
			line:Show()
		end
	elseif self.type == 8 then
		-- profession
		for i=2,#self.data do
			local spellID = self.data[i]
			local line = SourceFrame_MoreInfo_CreateLine(i-1)
			local spellName = GetSpellInfo(spellID)
			line.text:SetText(spellName)
			line.link.link = "http://wowhead.com/spell="..spellID
			line.textFrame.type = 2
			line.textFrame.data = "spell:"..spellID
			line:Show()
		end
	elseif self.type == 7 then
		-- achievement
		for i=2,#self.data do
			local a4ivID = self.data[i]
			local line = SourceFrame_MoreInfo_CreateLine(i-1)
			local _,a4ivName = GetAchievementInfo(a4ivID)
			line.text:SetText(a4ivName)
			line.link.link = "http://wowhead.com/achievement="..a4ivID
			line.textFrame.type = 2
			line.textFrame.data = GetAchievementLink(a4ivID)
			line:Show()
		end

	end
	for i=#self.data,#sourcesFrame.moreInfo.lines do
		sourcesFrame.moreInfo.lines[i]:Hide()
	end
	sourcesFrame.moreInfo:SetHeight(20+(#self.data-1)*25)

	sourcesFrame.moreInfo:Show()
end

sourcesFrame.lines = {}
local function SourceFrame_CreateLine(i)
	if sourcesFrame.lines[i] then
		return sourcesFrame.lines[i]
	end
	local line = CreateFrame("Frame",nil,sourcesFrame)
	sourcesFrame.lines[i] = line
	line:SetPoint("TOPLEFT",10,-30-(i-1)*25-25)
	line:SetSize(1,20)
	
	line.itemName = line:CreateFontString(nil,"ARTWORK","GameFontWhite")
	line.itemName:SetPoint("LEFT",0,0)
	line.itemName:SetSize(180,20)
	line.itemName:SetJustifyH("LEFT")
	line.itemName:SetJustifyV("MIDDLE")
	
	line.itemNameFrame = CreateFrame("Button",nil,line)
	line.itemNameFrame:SetAllPoints(line.itemName)
	line.itemNameFrame:SetScript("OnEnter",SourcesFrame_ItemName_OnEnter)
	line.itemNameFrame:SetScript("OnLeave",SourcesFrame_HideTooltip)
	line.itemNameFrame:SetScript("OnClick",SourcesFrame_ItemName_OnClick)
	
	line.itemLink = CreateFrame("Button",nil,line,"UIPanelButtonTemplate")
	line.itemLink:SetPoint("LEFT",line.itemName,"RIGHT",5,0)
	line.itemLink:SetSize(50,20)
	line.itemLink:SetText("link")
	line.itemLink:SetScript("OnClick",SourcesFrame_ItemLink_OnClick)

	line.sourceName = line:CreateFontString(nil,"ARTWORK","GameFontWhite")
	line.sourceName:SetPoint("LEFT",line.itemLink,"RIGHT",5,0)
	line.sourceName:SetSize(95,20)
	line.sourceName:SetJustifyH("LEFT")
	line.sourceName:SetJustifyV("MIDDLE")
	
	line.sourceData = line:CreateFontString(nil,"ARTWORK","GameFontWhite")
	line.sourceData:SetPoint("LEFT",line.sourceName,"RIGHT",5,0)
	line.sourceData:SetSize(280,20)
	line.sourceData:SetJustifyH("LEFT")
	line.sourceData:SetJustifyV("MIDDLE")
	
	line.sourceDataFrame = CreateFrame("Frame",nil,line)
	line.sourceDataFrame:SetAllPoints(line.sourceData)
	line.sourceDataFrame:SetScript("OnEnter",SourcesFrame_SourceData_OnEnter)
	line.sourceDataFrame:SetScript("OnLeave",SourcesFrame_HideTooltip)
	
	line.moreInfo = CreateFrame("Button",nil,line,"UIPanelButtonTemplate")
	line.moreInfo:SetPoint("LEFT",line.sourceData,"RIGHT",5,0)
	line.moreInfo:SetSize(80,20)
	line.moreInfo:SetText("more info")
	line.moreInfo:SetScript("OnClick",SourcesFrame_MoreInfo_OnClick)
	
	return line
end
sourcesFrame.moreInfo.lines = {}
function SourceFrame_MoreInfo_CreateLine(i)
	if sourcesFrame.moreInfo.lines[i] then
		return sourcesFrame.moreInfo.lines[i]
	end
	local line = CreateFrame("Frame",nil,sourcesFrame.moreInfo)
	sourcesFrame.moreInfo.lines[i] = line
	line:SetPoint("TOPLEFT",10,-10-(i-1)*25)
	line:SetSize(1,20)
	
	line.text = line:CreateFontString(nil,"ARTWORK","GameFontWhite")
	line.text:SetPoint("LEFT",0,0)
	line.text:SetSize(150,20)
	line.text:SetJustifyH("LEFT")
	line.text:SetJustifyV("MIDDLE")
	
	line.textFrame = CreateFrame("Frame",nil,line)
	line.textFrame:SetAllPoints(line.text)
	line.textFrame:SetScript("OnEnter",SourcesFrame_MoreInfo_Text_OnEnter)
	line.textFrame:SetScript("OnLeave",SourcesFrame_HideTooltip)
	
	line.link = CreateFrame("Button",nil,line,"UIPanelButtonTemplate")
	line.link:SetPoint("LEFT",line.text,"RIGHT",5,0)
	line.link:SetSize(50,20)
	line.link:SetText("link")
	line.link:SetScript("OnClick",SourcesFrame_ItemLink_OnClick)
	
	return line
end

local function Model_OnClick(self,button)
	if IsModifiedClick("DRESSUP") and self.Equipped then
		DressUpItemLink(self.Equipped)
		return
	elseif IsModifiedClick("CHATLINK") and self.Equipped then
		local _,link = GetItemInfo(self.Equipped)
		if link then
			if ChatEdit_GetActiveWindow() then
				ChatEdit_InsertLink(link)
			else
				ChatFrame_OpenChat(link)
			end
		end
		return
	end

	local c = #self.data
	sourcesFrame:SetHeight(40+c*25+25)
	sourcesFrame.TextLine:SetText(self.isCollected and "|cff00ff00Collected|r" or "|cffff0000Not collected|r")
	for i=1,c do
		local line = SourceFrame_CreateLine(i)
		
		local source = self.data[i]
		
		local name,link = GetItemInfo(source[2])
		line.itemName:SetText(link)
		line.itemLink.link = "http://wowhead.com/item="..source[1]..(source[6] and "&bonus="..source[6] or "")
		line.sourceName:SetText(SourceTypeNames[ source[4] ] or "")
		if source[7] then
			line.sourceName:SetTextColor(0,1,0,1)
		else
			line.sourceName:SetTextColor(1,1,1,1)
		end
		line.itemNameFrame.link = link
		line.moreInfo:Show()
		line.moreInfo.itemID = source[1]
		
		if source[4] == 3 and #source[5] > 1 then
			line.moreInfo.data = source[5]
			line.moreInfo.itemID = source[1]
			line.moreInfo.type = 3
			
			local npcID = source[5][2]
			line.sourceData:SetText(VendorNames[npcID])
			line.sourceDataFrame.type = nil
			local costTable = ItemCost[ source[1] ]
			if costTable then
				costTable = costTable[npcID] or costTable
				line.sourceDataFrame.type = 3
				line.sourceDataFrame.data = costTable
				if costTable[1] == 3 then
					for j=2,#costTable,2 do
						GetItemInfo(costTable[j])
					end
				end
			end
		elseif source[4] == 8 and #source[5] > 1 then
			line.moreInfo.data = source[5]
			line.moreInfo.type = 8
			
			local SpellName = GetSpellInfo(source[5][2])
			line.sourceData:SetText(SpellName)
			line.sourceDataFrame.type = 8
			line.sourceDataFrame.link = "spell:"..source[5][2]
		elseif source[4] == 7 and #source[5] > 1 then
			line.moreInfo.data = source[5]
			line.moreInfo.type = 7
			
			local _,a4ivName = GetAchievementInfo(source[5][2])
			line.sourceData:SetText(a4ivName or "")
			line.sourceDataFrame.type = 7
			line.sourceDataFrame.link = GetAchievementLink(source[5][2])
		elseif source[4] == 2 and #source[5] > 1 then
			line.sourceDataFrame.quests = source[5]
			local setted = false
			for j=2,#source[5] do
				inspectScantip:SetHyperlink("quest:"..source[5][j])
				if not setted and inspectScantip:NumLines() > 0 then
					local tooltipLine = _G["LegionWardrobeScanningTooltipTextLeft1"]
					local text = tooltipLine:GetText()
					if text and text ~= "" then
						line.sourceData:SetText(text)
						setted = true
					end
				end
				inspectScantip:ClearLines()
			end
			if not setted then
				line.sourceData:SetText("")
			end
			line.sourceDataFrame.sourceType = SourceTypeNames[ source[4] ] or ""
			line.sourceDataFrame.type = 2
			
			line.moreInfo.data = source[5]
			line.moreInfo.type = 2
		elseif source[4] == 1 and #source[5] > 1 then
			local instanceType,instanceID,bossID,diffList = source[5][2]:match("^(%d+)#(%d+)#(%d+)#(.*)")
			instanceType = tonumber(instanceType)
			instanceID = tonumber(instanceID)
			bossID = tonumber(bossID)
			
			line.sourceData:SetText(InstanceList[instanceID]..": "..BossList[bossID])
			
			line.sourceDataFrame.instanceType = instanceType
			line.sourceDataFrame.instanceID = instanceID
			line.sourceDataFrame.bossID = bossID
			line.sourceDataFrame.sourceType = SourceTypeNames[ source[4] ] or ""
			line.sourceDataFrame.diffList = {strsplit(":", diffList)}
			line.sourceDataFrame.type = 1
			
			line.moreInfo.data = source[5]
			line.moreInfo.type = 1
		else
			line.sourceData:SetText("")
			line.sourceDataFrame.type = nil
			line.moreInfo:Hide()
		end
		line:Show()
	end
	for i=c+1,#sourcesFrame.lines do
		sourcesFrame.lines[i]:Hide()
	end
	sourcesFrame.moreInfo:Hide()
	sourcesFrame:Show()
end

local function Model_OnModelLoaded(self)
	if self.cameraID then
		Model_ApplyUICamera(self, self.cameraID)
	end
end

local function Model_OnShow(self)
	self:SetUnit("player")
	self:Undress()
	if self.Equipped then
		self:TryOn(self.Equipped)
	end
	if self.cameraID then
		Model_ApplyUICamera(self, self.cameraID)
	end
end

local function Model_OnEnter(self)
	if not self.data then return end
	self.border:SetBackdropBorderColor(1, 1, 1, 1)
	if mainFrame.bigModel:IsShown() and self.Equipped then
		mainFrame.bigModel:Undress()
		mainFrame.bigModel:TryOn(self.Equipped)
	end
	GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
	GameTooltip:AddLine(self.isCollected and "|cff00ff00Collected|r" or "|cffff0000Not collected|r")
	for i=1,#self.data do
		local data = self.data[i]
		local name,link = GetItemInfo(data[2])
		GameTooltip:AddDoubleLine(link or " ",(data[7] and "|cff00ff00" or "").. (SourceTypeNames[ data[4] ] or "").."|r")
	end
	if self.isCollected == 2 then
		GameTooltip:AddLine("Collected by |cffffffff"..VLTW._Collected[self.visualID]:gsub(":",", "):gsub("%-"..realmKey,""), nil, nil, nil, true)
	end
	GameTooltip:Show()
end

local function Model_OnLeave(self)
	if self.isCollected then
		self.border:SetBackdropBorderColor(1, 1, 0.5, 1)
	else
		self.border:SetBackdropBorderColor(0.6, 0.6, 0.6, .8)
	end
	GameTooltip:Hide()
end

local function Model_Collected_SetView(self,isYellow)
	if isYellow then
		self:SetTexture([[Interface\AddOns\LegionWardrobe\Transmogrify]])
		self:SetTexCoord(222/512,250/512,127/512,153/512)
	else
		self:SetTexture("Interface\\RaidFrame\\ReadyCheck-Ready")
		self:SetTexCoord(0,1,0,1)
	end
end

local models = {}
mainFrame.models = models
for i=0,2 do
	for j=0,5 do
		local borderFrame = CreateFrame("Frame",nil,mainFrame)
	
		local model = CreateFrame("DressUpModel",nil,mainFrame)
		models[i*6+j+1] = model
		model:SetPoint("TOPLEFT",50+100*j,-100-140*i)
		model:SetSize(90,130)
		
		model:SetScript("OnModelLoaded",Model_OnModelLoaded)
		model:SetScript("OnShow",Model_OnShow)
		model:SetScript("OnEnter",Model_OnEnter)
		model:SetScript("OnLeave",Model_OnLeave)
		model:SetScript("OnMouseUp",Model_OnClick)
		
		model.collected = model:CreateTexture(nil,"OVERLAY")
		model.collected:SetPoint("CENTER",model,"TOPLEFT",10,-5)
		model.collected:SetSize(30,30)
		model.collected:SetTexture("Interface\\RaidFrame\\ReadyCheck-Ready")
		
		model.collected.SetView = Model_Collected_SetView
		
		borderFrame:SetPoint("TOPLEFT",model,-4,4)
		borderFrame:SetPoint("BOTTOMRIGHT",model,4,-4)
		
		borderFrame:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",tile = true,tileSize = 1,edgeSize = 8,insets = {left = 2,right = 2,top = 2,bottom = 2}})
		borderFrame:SetBackdropColor(0, 0, 0, 0.8) 
		borderFrame:SetBackdropBorderColor(0.6, 0.6, 0.6, 1)
		
		model.border = borderFrame

	end
end

local PageText = mainFrame:CreateFontString(nil,"ARTWORK","GameFontWhite")
PageText:SetPoint("BOTTOM",-57,53)
PageText:SetJustifyH("RIGHT")

local PrevPageButton = CreateFrame("Button",nil,mainFrame)
PrevPageButton:SetPoint("BOTTOM",20,43)
PrevPageButton:SetSize(32,32)
PrevPageButton:SetNormalTexture("Interface\\Buttons\\UI-SpellbookIcon-PrevPage-Up") 
PrevPageButton:SetPushedTexture("Interface\\Buttons\\UI-SpellbookIcon-PrevPage-Down") 
PrevPageButton:SetDisabledTexture("Interface\\Buttons\\UI-SpellbookIcon-PrevPage-Disabled") 
PrevPageButton:SetHighlightTexture("Interface\\Buttons\\UI-Common-MouseHilight","ADD") 
PrevPageButton:SetScript("OnClick",function ()
	currentPage = max(1,currentPage - 1)
	UpdatePage()
end)

local NextPageButton = CreateFrame("Button",nil,mainFrame)
NextPageButton:SetPoint("BOTTOM",55,43)
NextPageButton:SetSize(32,32)
NextPageButton:SetNormalTexture("Interface\\Buttons\\UI-SpellbookIcon-NextPage-Up") 
NextPageButton:SetPushedTexture("Interface\\Buttons\\UI-SpellbookIcon-NextPage-Down") 
NextPageButton:SetDisabledTexture("Interface\\Buttons\\UI-SpellbookIcon-NextPage-Disabled") 
NextPageButton:SetHighlightTexture("Interface\\Buttons\\UI-Common-MouseHilight","ADD")
NextPageButton:SetScript("OnClick",function ()
	currentPage = min(currentMaxPages,currentPage + 1)
	UpdatePage()
end)

mainFrame:SetScript("OnMouseWheel",function (self,delta)
	if delta < 0 and NextPageButton:IsEnabled() then
		NextPageButton:Click()
	elseif delta > 0 and PrevPageButton:IsEnabled() then
		PrevPageButton:Click()
	end
end)

function UpdatePage()
	PageText:SetFormattedText(COLLECTION_PAGE_NUMBER, currentPage, currentMaxPages)
	if currentPage == 1 then PrevPageButton:Disable() else PrevPageButton:Enable() end
	if currentPage >= currentMaxPages then NextPageButton:Disable() else NextPageButton:Enable() end

	local pos = (currentPage - 1) * #models + 1
	local m_c = 0
	
	local categoryData = AllData[currentCategory]
	if IsFilterEnabled then
		categoryData = AllData[-1]
	end
	
	for i=pos,#categoryData do
		m_c = m_c + 1
		local model = models[m_c]
		model:Show()
		model.border:Show()
		model:Undress()
		
		model.data = {}
		local visualID = categoryData[i]
		model.visualID = visualID
		local visualData = Visuals[ visualID ]
		local isCollected = 0
		if visualData then
			for k=1,#visualData do
				local item = visualData[k]
				local itemID = type(item) == 'number' and item or tonumber(item:match("^(%d+)"))
				local itemLink = "item:"..itemID
				
				local sourceData = ItemSource[item]
				local sourceType = sourceData[1]
				
				if type(item) == 'string' then
					local count = 0
					for j=1,#item do
						if item:sub(j,j) == ":" then
							count = count + 1
						end
					end
					itemLink = itemLink .. ":0:0:0:0:0:0:0:"..UnitLevel'player'..":0:0:0:"..count..":"..item:gsub("^%d+:","")
				end
				
				GetItemInfo(itemID)	--Cache it
				
				local currItemCollected = false
				if IsItemCollected(item) then
					isCollected = 1
					currItemCollected = true
				elseif sourceType == 2 then
					for j=2,#sourceData do
						if CharQuestsData[ sourceData[j] ] then
							isCollected = 1
							currItemCollected = true
							break
						end
					end
				end
				
				model.data[k] = {
					itemID,
					itemLink,
					item,
					sourceType,
					sourceData,
					type(item) == 'string' and item:gsub("^%d+:",""),
					currItemCollected,
				}
				
				if k == 1 then
					model:TryOn(itemLink)
					model.Equipped = itemLink
				end
			end
		else
			--print('Visual',visualID,'not found')
		end
		if isCollected == 0 and VLTW._Collected[visualID] and not VLTW._Disable then
			isCollected = 2
		end
		if isCollected > 0 then
			model.collected:Show()
			model.collected:SetView(isCollected == 2)
			model.border:SetBackdropBorderColor(1, 1, 0.5, 1)
		else
			model.collected:Hide()
			model.border:SetBackdropBorderColor(0.6, 0.6, 0.6, .8)
		end
		model.isCollected = isCollected > 0 and isCollected or nil
		if m_c >= #models then
			break
		end
	end
	for i=m_c+1,#models do
		models[i]:Hide()
		models[i].border:Hide()
	end
	if IsUsedZeroCam then
		UpdateCamera0()
	end
end

local isFirstLoad = true

local function UpdateCollectCategory(categoryID)
	local collected = 0
	
	local categoryData = AllData[categoryID]
	if IsFilterEnabled then
		categoryData = AllData[-1]
	end
	
	for i=1,#categoryData do
		local visualID = categoryData[i]
		local visualData = Visuals[ visualID ]
		local isCollected = 0
		if visualData then
			for k=1,#visualData do
				local item = visualData[k]
				local sourceData = ItemSource[item]
				if IsItemCollected(item) then
					isCollected = 1
				elseif sourceData[1] == 2 then
					for j=2,#sourceData do
						if CharQuestsData[ sourceData[j] ] then
							isCollected = 1
							break
						end
					end
				end
			end
		end
		if not VLTW._Disable then
			local inDBstr = VLTW._Collected[visualID]
			if isCollected == 1 then
				if not inDBstr then
					VLTW._Collected[visualID] = charKey
				elseif not inDBstr:find(charKeyFind) then
					VLTW._Collected[visualID] = inDBstr..":"..charKey
				end
			elseif inDBstr then
				isCollected = 1
				if inDBstr:find(charKeyFind) then
					inDBstr = strtrim(inDBstr:gsub(":*"..charKeyFind..":*",":"),":")
					if inDBstr == "" then
						VLTW._Collected[visualID] = nil
						isCollected = 0
					else
						VLTW._Collected[visualID] = inDBstr
					end
				end
			end
		end
		collected = collected + isCollected
	end
	return collected
end

local function CreateFilteredCategory(categoryID)
	local categoryData = AllData[currentCategory]
	local filteredData = {}
	for i=1,#categoryData do
		local visualID = categoryData[i]
		local visualData = Visuals[ visualID ]
		local isFiltered = false
		if visualData then
			for k=1,#visualData do
				local item = visualData[k]				
				local sourceData = ItemSource[item]
				local sourceType = sourceData[1]
				
				if bit.band(SourceTypeBitFilters[sourceType] or SourceTypeBitFilters[0], FilterBits) > 0 then
					isFiltered = true
					break
				end
			end
		end
		if isFiltered then
			filteredData[#filteredData + 1] = visualID
		end
	end
	return filteredData
end

function UpdateCategory(categoryID)
	currentCategory = categoryID
	
	for i=1,#models do
		local model = models[i]
		model.cameraID = CameraData[categoryID]
		model:SetUnit("player")
		model:Undress()
		Model_ApplyUICamera(model, CameraData[categoryID])
	end
	mainFrame.bigModel.cameraID = CameraData[categoryID]
	mainFrame.bigModel.itemID = CategoryBigModelItem[categoryID]
	mainFrame.bigModel:Reload()
	
	IsUsedZeroCam = false
	
	local total = #AllData[categoryID]
	
	if IsFilterEnabled then
		AllData[-1] = CreateFilteredCategory(categoryID)
		total = #AllData[-1]
	end
	
	currentPage = 1
	currentMaxPages = ceil(total / #models)
	UpdatePage()
	if isFirstLoad then
		C_Timer.NewTimer(1,function() UpdatePage() end)
	end
	isFirstLoad = nil
	
	local collected = UpdateCollectCategory(categoryID)
	ProgressBar:SetMinMaxValues(0, total)
	ProgressBar:SetValue(collected)
	ProgressBar.text:SetFormattedText(HEIRLOOMS_PROGRESS_FORMAT, collected, total)

end

local function FindBonuses(link)
	if link then 
		local bonusesNum,bonusesString = link:match("item:%d+:[0-9%-]*:[0-9%-]*:[0-9%-]*:[0-9%-]*:[0-9%-]*:[0-9%-]*:[0-9%-]*:[0-9%-]*:[0-9%-]*:[0-9%-]*:[0-9%-]*:([0-9%-]*):([0-9:]*)")
		if bonusesNum and bonusesString and tonumber(bonusesNum) then
			local res = {}
			for i=1,tonumber(bonusesNum) do
				local bonus = bonusesString:match("^[0-9%-]+")
				if bonus then
					bonus = tonumber(bonus)
					if majorBonuses[bonus] then
						return bonus
					end
				end
				if bonusesString:find(":") then
					bonusesString = bonusesString:gsub("^[^:]*:","")
				end
			end
		end
	end
	return 0
end
local function table_add(t1,t2)
	for i=1,#t2 do
		t1[#t1+1] = t2[i]
	end
end
local function string_add_or_create(str,toadd)
	if type(str) ~= "string" then
		str = ""
	end
	if not str:find(toadd) then
		str = str .. toadd
	end
	return str
end

local function UpdateBagsData()
	wipe(VLTW[charKey].bags)
	for i=0,4 do
		for j=1,GetContainerNumSlots(i) do
			local link = GetContainerItemLink(i, j)
			if link then
				local itemID = tonumber(link:match("item:(%d+)"))
				if ItemsCheckBonuses[itemID] then
					local bonus = FindBonuses(link)
					
					VLTW[charKey].bags[itemID] = string_add_or_create(VLTW[charKey].bags[itemID],"#"..bonus.."#")
				end
			end
		end
	end
	for _,slot in ipairs {1,2,3,15,5,9,10,6,7,8,11,12,13,14,16,17} do
		local link = GetInventoryItemLink('player',slot)
		if link then
			local itemID = tonumber(link:match("item:(%d+)"))
			if ItemsCheckBonuses[itemID] then
				local bonus = FindBonuses(link)
				
				VLTW[charKey].bags[itemID] = string_add_or_create(VLTW[charKey].bags[itemID],"#"..bonus.."#")
			end
		end
	end
end

mainFrame:SetScript("OnShow",function(self)
	self:SetScript("OnShow",nil)
	
	CharQuestsData = GetQuestsCompleted()
	
	LoadAddOn("LegionWardrobeData")
	if _G.LTSData then
		local LTS = _G.LTSData
	
		AllData = LTS.AllData
		CamerasData = LTS.CamerasData
		Visuals = LTS.Visuals
		ItemSource = LTS.ItemSource
		VendorNames = LTS.VendorNames
		InstanceList = LTS.InstanceList
		BossList = LTS.BossList
		DiffList = LTS.DiffList
		ItemCost = LTS.ItemCost
		ItemsCheckBonuses = LTS.ItemsCheckBonuses
		
		AllData = AllData[playerClass]	
		
		LTSData = nil
		
		eventsFrame:RegisterEvent("BANKFRAME_OPENED")
		eventsFrame:RegisterEvent("VOID_STORAGE_OPEN")
	else
		self:SetScript("OnShow",function()
			print("<Legion Wardrobe> Error: Legion Wardrobe Data isn't loaded")
			self:Hide()
		end)
		print("<Legion Wardrobe> Error: Legion Wardrobe Data isn't loaded")
		self:Hide()
		return
	end
	
	UpdateBagsData()
	
	if not VLTW._Disable then
		for i=1,#CategoriesSlotsByClass do
			UpdateCollectCategory(CategoriesSlotsByClass[i])
		end
	end
	
	UpdateCategory(CategoriesSlotsByClass[1])
end)

function IsItemCollected(itemID)
	local sItemID
	if type(itemID) == 'number' then
		sItemID = itemID
	else
		sItemID = tonumber(itemID:match("^(%d+)"))
	end
	local isCollected = false
	if not ItemsCheckBonuses[sItemID] then
		isCollected = GetItemCount(itemID,true) > 0
		if not isCollected and VLTW[charKey].vs[sItemID] then
			isCollected = true
		end
	else
		local bonusList
		if type(itemID) == 'number' then
			bonusList = {"#0#"}
		else
			bonusList = {strsplit(":", itemID:match("^%d+:(.+)"),nil)}
			for i=1,#bonusList do
				bonusList[i] = "#"..bonusList[i].."#"
			end
		end
		local bags,bank,vs = VLTW[charKey].bags[sItemID], VLTW[charKey].bank[sItemID], VLTW[charKey].vs[sItemID]
		for i=1,#bonusList do
			if bags and bags:find(bonusList[i]) then
				isCollected = true
			end
			if bank and bank:find(bonusList[i]) then
				isCollected = true
			end
			if vs and vs:find(bonusList[i]) then
				isCollected = true
			end
		end
	end
	
	return isCollected
end

eventsFrame = CreateFrame("Frame")
eventsFrame:RegisterEvent("ADDON_LOADED")
eventsFrame:SetScript("OnEvent",function(self,event,arg1)
	if event == "ADDON_LOADED" then
		if arg1 == GlobalAddonName then
			if type(_G.VLTW) ~= 'table' then
				_G.VLTW = {}
				print("You have installed <Legion Wardrobe> addon. Type \"/wardrobe\" or use button in Collections Journal to open it.")
			end
			VLTW = _G.VLTW
			if not VLTW[charKey] then
				VLTW[charKey] = {
					bags = {},
					bank = {},
					vs = {},
				}
			end
			
			if not VLTW._Collected then
				VLTW._Collected = {}
			end
			
			CameraDataOffset = CamerasRace[playerRace..playerSex] or 0
			
			UpdateAlerts(not VLTW[charKey].visited_VS,not VLTW[charKey].visited_Bank)
			
			if playerClass == "ALL" then
				VLTW._Disable = true
			end
			
			mainFrame.disableOtherChars:SetChecked(VLTW._Disable)
			mainFrame.disableAlerts:SetChecked(VLTW._AlertsOff)
		elseif arg1 == "Blizzard_Collections" then
			local button = CreateFrame("Button","CollectionsJournalTab5USER",CollectionsJournal,"CollectionsJournalTab")
			button:SetID(5)
			button:SetPoint("LEFT",CollectionsJournalTab4,"RIGHT",-16,0)
			button:SetText("Wardrobe")
			button:SetScript("OnClick",function()
				CollectionsJournal.CloseButton:Click()
				mainFrame:Show()
			end)
			
			PanelTemplates_DeselectTab(button)
		end
	elseif event == "BANKFRAME_OPENED" then
		VLTW[charKey].visited_Bank = true
		mainFrame:Hide()
		wipe(VLTW[charKey].bank)
		for i=5,12 do
			local bug = i==12 and -1 or i
			for j=1,GetContainerNumSlots(bug) do
				local link = GetContainerItemLink(bug, j)
				if link then
					local itemID = tonumber(link:match("item:(%d+)"))
					if ItemsCheckBonuses[itemID] then
						local bonus = FindBonuses(link)
						
						VLTW[charKey].bank[itemID] = string_add_or_create(VLTW[charKey].bank[itemID],"#"..bonus.."#")
					end
				end
			end
		end
		UpdateAlerts(not VLTW[charKey].visited_VS,not VLTW[charKey].visited_Bank)
		if not VLTW._Disable then
			for i=1,#CategoriesSlotsByClass do
				UpdateCollectCategory(CategoriesSlotsByClass[i])
			end
		end
		self:UnregisterEvent("BANKFRAME_OPENED")
	elseif event == "VOID_STORAGE_OPEN" then
		VLTW[charKey].visited_VS = true
		mainFrame:Hide()
		C_Timer.NewTimer(1,function() 
			wipe(VLTW[charKey].vs)
			for i=1,160 do
				local link = GetVoidItemHyperlinkString(i)
				if link then
					local itemID = tonumber(link:match("item:(%d+)"))
					if ItemsCheckBonuses[itemID] then
						local bonus = FindBonuses(link)
						
						VLTW[charKey].vs[itemID] = string_add_or_create(VLTW[charKey].vs[itemID],"#"..bonus.."#")
					else
						VLTW[charKey].vs[itemID] = true
					end
				end
			end
		end)
		UpdateAlerts(not VLTW[charKey].visited_VS,not VLTW[charKey].visited_Bank)
		if not VLTW._Disable then
			for i=1,#CategoriesSlotsByClass do
				UpdateCollectCategory(CategoriesSlotsByClass[i])
			end
		end
		self:UnregisterEvent("VOID_STORAGE_OPEN")
	end
end)

SlashCmdList["LTWardrobe"] = function() 
	mainFrame:Show() 
end
SLASH_LTWardrobe1 = "/wardrobe"